/*
** Module   : XNodeRefImpl
** Abstract : DOM XML Node reference resource type implementation.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- -----------------------Description------------------------
** 001 EVL 20130204 Created initial version. Major rework to use w3c backend.
** 002 CA  20130221 Added LOCAL-NAME support. 
** 003 EVL 20130222 Adding NODE-VALUE attribute and SET-ATTRIBUTE-NODE method.
*/

package com.goldencode.p2j.xml;

import org.w3c.dom.*;
import com.goldencode.p2j.util.*;

/**
 * Implementation of the X-Node-Reference specific features of the Progress
 * DOM XML support objects.
 */
public class XNodeRefImpl
extends XEntityImpl
implements XNodeRef
{

   public static final String RESOURCE_TYPE = "X-NODEREF";

   /**
    * Default constructor.
    */
   public XNodeRefImpl()
   {
      super();
      setResourceType();
   }

   /**
    * Special constructor accepting the namspace URI specification to make
    * another node reference case.
    *
    * @param    node
    *           The node associated with this node.
    */
   public XNodeRefImpl(Node node)
   {
      super(node);
      setResourceType();
   }

   private void setResourceType()
   {
      setResourceType(RESOURCE_TYPE);
   }

   // Attributes
   /**
    * Getting the attribute names available for current node reference.
    * 
    * @return   The comma separated character string of all available
    *           attributed.
    */
   @Override
   public character getAttributeNames()
   {
      if (unknown() || !valid())
      {
         return new character();
      }
      final StringBuilder sb = new StringBuilder();
      final NamedNodeMap nnm = getNode().getAttributes();
      final int iSize = nnm.getLength();
      for (int i = 0; i < iSize; i++)
      {
         sb.append(((Attr)nnm.item(i)).getName());
         if (i < iSize - 1)
         {
            sb.append(',');
         }
      }   

      return new character(sb.toString());
   }
   
   /**
    * Getting the value of the XML node value attribute.
    * 
    * @return   The character value of the node reference.
    */
   @Override
   public character getNodeValue()
   {
      final character result;

      if (unknown() || !valid())
      {
         result = new character();
      }
      else
      {
         result = new character(getNode().getNodeValue());
      }
      return result;
   }
   
   /**
    * Setting the value of the XML node value attribute.
    * 
    * @param    value
    *           The new value of the node value attribute.
    */
   @Override
   public void setNodeValue(character value)
   {
      if (unknown() || !valid())
      {
         return;
      }
      final String toSetValue;

      if (value.isUnknown())
      {
         toSetValue = null;
      }
      else
      {
         toSetValue = value.toStringMessage();
      }
      getNode().setNodeValue(toSetValue);
   }
   
   /**
    * Setting the value of the XML node value attribute.
    * 
    * @param    value
    *           The new value of the node value attribute.
    */
   @Override
   public void setNodeValue(String value)
   {
      setNodeValue(new character(value));
   }

   /**
    * Creating the new node within the current Document object.
    *
    * @param xNodeRef The valid handle class instance to use for the cloned XML node.
    * @param deep     A logical that if <code>true</code> specifies that the whole sub-trees is to be
    *                 copied. The default value is {@code false}.
    *
    * @return The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @Override
   public logical cloneNode(handle xNodeRef, logical deep)
   {
      if (unknown() || !valid())
      {
         return new logical(false);
      }

      final Node node = getNode();
      final Node clonedNode = node.cloneNode(deep.booleanValue());
      xNodeRef.assign(new XNodeRefImpl(clonedNode));

      return xNodeRef.isValid();
   }

   /**
    * Getting the Document the current reference belongs to.
    *
    * @return   The handle of the Document containing tne current reference..
    */
   @Override
   public handle getOwnerDocument()
   {
      return getHandle();
   }

   /**
    * Getting the Parent of current node.
    *
    * @param    hNoderefParent
    *           A valid handle to assign parent XML node.
    *
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   @Override
   public logical getParent(handle hNoderefParent)
   {
      if (unknown() || !valid() || hNoderefParent == null)
      {
         return new logical(false);
      }
      final Node node = getNode().getParentNode();

      hNoderefParent.assign(new XNodeRefImpl(node));

      return hNoderefParent.isValid();
   }

   // Methods
   /**
    * Deleting the internals of the current X-Node-Reference.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   @Override
   public logical deleteNode()
   {
      if (unknown() || !valid())
      {
         return new logical(false);
      }
      final Node node = getNode();
      final Node removedNode = node.getOwnerDocument().removeChild(node);

      return new logical(removedNode != null);
   }
   
   /**
    * Getting the attribute value for a given attribute name.
    * 
    * @param    name
    *           The name of the attribute to return.
    *
    * @return   The comma separated character string of all available
    *           attributed.
    */
   @Override
   public character getAttribute(String name)
   {
      return getAttribute(new character(name));
   }
   
   /**
    * Getting the attribute value for a given attribute name.
    * 
    * @param    name
    *           The name of the attribute to return.
    *
    * @return   Value as a character string, or the empty string
    *           if that attribute does not have a specified or default value.
    */
   @Override
   public character getAttribute(character name)
   {
      if (unknown() || !valid() || name.isUnknown())
      {
         return new character();
      }
      final character result;
      final Node node = getNode();

      if (node.getNodeType() == Node.ELEMENT_NODE)
      {
         final Element elementNode = (Element) node;
         result = new character(elementNode.getAttribute(name.toStringMessage()));
      }
      else
      {
         result = new character();
      }

      return result;
   }
   
   /**
    * Setting the attribute value for a given attribute name.
    * 
    * @param    name
    *           The name of the attribute to change.
    * @param    value
    *           The new value for the given attribute name.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   @Override
   public logical setAttribute(String name, String value)
   {
      return setAttribute(new character(name), new character(value));
   }
   
   /**
    * Setting the attribute value for a given attribute name.
    * 
    * @param    name
    *           The name of the attribute to change.
    * @param    value
    *           The new value for the given attribute name.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   @Override
   public logical setAttribute(String name, character value)
   {
      return setAttribute(new character(name), value);
   }
   
   /**
    * Setting the attribute value for a given attribute name.
    * 
    * @param    name
    *           The name of the attribute to change.
    * @param    value
    *           The new value for the given attribute name.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   @Override
   public logical setAttribute(character name, String value)
   {
      return setAttribute(name, new character(value));
   }
   
   /**
    * Setting the attribute value for a given attribute name.
    * 
    * @param    name
    *           The name of the attribute to change.
    * @param    value
    *           The new value for the given attribute name.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @Override
   public logical setAttribute(character name, character value)
   {
      if (unknown() || !valid() || name.isUnknown())
      {
         return new logical(false);
      }

      final Node node = getNode();
      final logical result;
      if (node.getNodeType() == Node.ELEMENT_NODE)
      {
         final Element elementNode = (Element) node;
         elementNode.setAttribute(name.toStringMessage(), value.toStringMessage());
         result = new logical(true);
      }
      else
      {
         result = new logical(false);
      }

      return result;
   }

   /**
    * Associates XML attribute node with the given X-Noderef object handle.
    * 
    * @param    handle
    *           The handle that represents an XML attribute node created with create node or
    *           create node namespace.
    *
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @Override
   public logical setAttributeNode(handle handle)
   {
      if (unknown() || !valid() || handle.isUnknown() || !handle._isValid())
      {
         return new logical(false);
      }
      final Object resource = handle.get();
      if (!(resource instanceof XNodeRefImpl))
      {
         return new logical(false);
      }
      final Node currentNode = getNode();
      final Node handleNode = ((XNodeRefImpl) resource).getNode();
      final logical result;
      if (currentNode.getNodeType() == Node.ELEMENT_NODE &&
              handleNode.getNodeType() == Node.ATTRIBUTE_NODE)
      {
         final Element elementNode = (Element) currentNode;
         elementNode.setAttributeNode((Attr) handleNode);
         result = new logical(true);
      }
      else
      {
         result = new logical(false);
      }

      return result;
   }

   @Override
   public logical getAttributeNode(handle handle, character attributeName)
   {
      if (unknown() || !valid() || attributeName.isUnknown())
      {
         return new logical(false);
      }
      final Node node = getNode();
      final logical result;
      if (node.getNodeType() == Node.ELEMENT_NODE)
      {
         final Element elementNode = (Element) node;
         Attr attrNode = elementNode.getAttributeNode(attributeName.toStringMessage());
         if (attrNode != null)
         {
            handle.assign(new XNodeRefImpl(attrNode));
            result = handle.isValid();
         }
         else
         {
            result = new logical(false);
         }
      }
      else
      {
         result = new logical(false);
      }
      return result;
   }

   /**
    * Setting the binary value of the XML node value attribute.
    *
    * @param value The new value of the node value attribute.
    *
    * @return The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @Override
   public logical longcharToNodeValue(longchar value)
   {
      if (unknown() || !valid())
      {
         return new logical(false);
      }

      final Node node = getNode();

      node.setTextContent(value.toStringMessage());
      return new logical(true);
   }

   /**
    * Setting the XML node value attribute to binary value of longchar.
    *
    * @param value To save the value on XML node to this parameter.
    *
    * @return The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @Override
   public logical nodeValueToLongchar(longchar value)
   {
      if (unknown() || !valid())
      {
         return new logical(false);
      }
      value.setValue(getNode().getNodeValue());

      return new logical(true);
   }

   /**
    * Normalizes TEXT and ATTRIBUTE nodes in the full depth of the sub-tree under this XML node.
    *
    * @return The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @Override
   public logical normalize()
   {
      if (unknown() || !valid())
      {
         return new logical(false);
      }

      final Node node = getNode();
      node.normalize();

      return new logical(true);
   }

   /**
    * Get the value of the LOCAL-NAME attribute.
    * 
    * @return   See above.
    */
   @Override
   public character getLocalName()
   {
      final String localName = getNode().getLocalName();

      if (localName != null)
      {
         return new character(localName);
      }
      else
      {
         return new character("");
      }
   }

   /**
    * Setter for NAMESPACE-PREFIX attribute.
    *
    * @param    prefix
    *           The new value of the NAMESPACE-PREFIX attribute.
    */
   @Override
   public void setNamespacePrefix(character prefix)
   {
      if (unknown() || !valid())
      {
         return;
      }

      final Node currentNode = getNode();
      final String prefixStr = prefix.getValue(); //null is ok here
      if (currentNode.getNodeType() == Node.ELEMENT_NODE)
      {
         final Element elementNode = (Element) currentNode;
         elementNode.setPrefix(prefixStr);
      }
      else if (currentNode.getNodeType() == Node.ATTRIBUTE_NODE)
      {
         final Attr attributeNode = (Attr) currentNode;
         attributeNode.setPrefix(prefixStr);
      }
   }
}
